# How to Develop and Use Libcaosdb

## Build

We use [cmake](https://cmake.org) as build tool.

1. `mkdir build && cd build/`
2. `cmake -B . ..`
3. `cmake --build .`

### Troubleshooting

#### protobufConfig.cmake not found

If you encounter something like `protobufConfig.cmake not found` when doing
`cmake -B . ..` your system doesn't have the gRPC library installed at all
or your installation doesn't have the required `protobufConfig.cmake`
file. In this case you can install grpcc manually by cloning the
repository and its submodules nad building it from the sources:

```sh
RUN git clone --recurse-submodules -b v1.38.1 https://github.com/grpc/grpc
cd grpc
mkdir -p cmake/build
cp cmake/build
cmake -DgRPC_INSTALL=ON \
      -DgRPC_BUILD_CSHARP_EXT=OFF \
      -DgRPC_BUILD_GRPC_CSHARP_PLUGIN=OFF \
      -DgRPC_BUILD_GRPC_PYTHON_PLUGIN=OFF \
      -DgRPC_BUILD_GRPC_RUBY_PLUGIN=OFF \
      -DgRPC_BUILD_GRPC_PHP_PLUGIN=OFF \
      -DgRPC_BUILD_GRPC_OBJECTIVE_C_PLUGIN=OFF \
      -DgRPC_BUILD_GRPC_NODE_PLUGIN=OFF \
      -DgRPC_BUILD_TESTS=OFF \
      ../..
make -j3  # Or whatever number of jobs suits your system
make install
```

It may make sense to specify a custom directory for your grpc
installation such as `$HOME/.local` by adding
`-DCMAKE_INSTALL_PREFIX=/path/to/install/dir` to the above `cmake`
command. This prevents interference with a possible system-wide
installation of grpc that's lacking the relevant cmake files.

Depending on your system, it may be necessary to manually install the
`abseil` submodule:

```sh
cd grpc
mkdir -p third_party/abseil-cpp/cmake/build
cd third_party/abseil-cpp/cmake/build
cmake -DCMAKE_POSITION_INDEPENDENT_CODE=TRUE ../..
make -j3
make install
```

As above, it may make sense to specify a `CMAKE_INSTALL_PREFIX` in
order to not interfere with system-wide installations.

You may have to repeat this for other submodules that haven't been
installed correctly by `make install` of `grpc`.

## Unit Tests

### Build

For the tests there is a slightly different setup required (with option `-D CMAKE_BUILD_TYPE=Debug`)

1. `mkdir build && cd build/`
2. `cmake -B . -D CMAKE_BUILD_TYPE=Debug ..`
3. `cmake --build .`

### Run

In the build directory, run `ctest`

### Troubleshooting

Depending on your gtest installation, there may be warnings treated as
errors during the above build command, e.g., a
`-Werror=maybe-uninitialized` in
`caosdb-cpplib-new/build/_deps/googletest-src/googletest/src/gtest-death-test.cc`. In
that case, suppress the corresponding warning by adding
`-DCMAKE_CXX_FLAGS="-Wno-maybe-uninitialized"` to the cmake command in 2.

### Framework

We use [GoogleTest](https://google.github.io/googletest/) for unit testing.

### Coverage

We use [gcov](https://gcc.gnu.org/onlinedocs/gcc/Gcov.html) and
[lcov](https://github.com/linux-test-project/lcov) for generating test coverage
reports.

In the build directory, generate the coverage report by running
`cmake --build . --target unit_test_coverage`.

Note that this special target will run the tests again. Thus it is not
necessary to run `ctest` prior to this target.

The coverage report can be viewed in a browser by opening
`<build_directory>/unit_test_coverage/index.html`.

## Code Formatting

* install clang-format on your system.
* `clang-format -i --verbose $(find test/ src/ include/ -type f -iname "*.cpp" -o -iname "*.h" -o -iname "*.h.in")`

## Documentation

In the build directory, run

* `cmake --build . --target doc-doxygen` (generate Doxygen)
* `cmake --build . --target doc-sphinx` (generate Sphinx)
